using DailyDigestWorker;
using DailyDigestWorker.Configuration; //  using    
using DailyDigestWorker.Services;
using Microsoft.Extensions.Options; //  IOptions
using Telegram.Bot; //  Telegram.Bot
using WTelegram; //  WTelegramClient
using System.Text;


Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);

// ---    Program.cs    ---

var builder = Host.CreateApplicationBuilder(args); //    ApplicationBuilder

// ---  ---
//    builder.Configuration ( appsettings.json, env vars, user secrets  ..)

//       POCO
//      IOptions<T>
builder.Services.Configure<BotConfiguration>(builder.Configuration.GetSection("BotConfiguration"));
builder.Services.Configure<TelegramClientSettings>(builder.Configuration.GetSection("TelegramClient"));
builder.Services.Configure<ApiKeys>(builder.Configuration.GetSection("ApiKeys"));
builder.Services.Configure<DataSourceUrls>(builder.Configuration.GetSection("DataSourceUrls"));
builder.Services.Configure<GeminiSettings>(builder.Configuration.GetSection("GeminiSettings"));
//      SchedulingSettings
builder.Services.Configure<SchedulingSettings>(builder.Configuration.GetSection("Scheduling"));


// ---   (Dependency Injection) ---

// 1.  HttpClientFactory (   API , , , Gemini)
builder.Services.AddHttpClient();

// 2.  Telegram Bot Client (  )
//     Singleton,        
builder.Services.AddSingleton<ITelegramBotClient>(sp => // sp - Service Provider
{
    //     IOptions
    var botConfig = sp.GetRequiredService<IOptions<BotConfiguration>>().Value;
    if (string.IsNullOrWhiteSpace(botConfig.BotToken))
    {
        //    ,     -    
        throw new InvalidOperationException("Bot Token    appsettings.json  .");
    }
    return new TelegramBotClient(botConfig.BotToken);
});

// 3.  WTelegramClient (  )
//  Singleton. WTelegramClient    .
builder.Services.AddSingleton(sp => //    Client
{
    var clientSettings = sp.GetRequiredService<IOptions<TelegramClientSettings>>().Value;
    var logger = sp.GetRequiredService<ILogger<WTelegram.Client>>(); //  

    //       WTelegramClient
    string? GetConfig(string configKey)
    {
        switch (configKey)
        {
            case "api_id": return clientSettings.ApiId.ToString();
            case "api_hash": return clientSettings.ApiHash;
            case "phone_number": return clientSettings.PhoneNumber;
            case "session_pathname": return clientSettings.SessionPath; // <---   CASE
                                                                        //      "password", "verification_code"  .
            default:
                logger.LogWarning("WTelegramClient    : {ConfigKey}", configKey);
                return null; //  null   
        }
    }
    // WTelegramClient     
    var client = new WTelegram.Client(GetConfig);

    //     WTelegramClient (,    )
    // WTelegram   Microsoft.Extensions.Logging,   
    //      , :
    // WTelegram.Helpers.Log = (level, message) => logger.Log((LogLevel)level, message);


    //     

    return client;
});

// 4.      
builder.Services.AddTransient<ICurrencyService, CbrCurrencyService>();
builder.Services.AddTransient<ICryptoService, CoinGeckoCryptoService>();
builder.Services.AddTransient<IWeatherService, OpenWeatherMapService>();

// 5.      
builder.Services.AddTransient<IDigestBuilderService, DigestBuilderService>();
builder.Services.AddTransient<ITelegramBotSender, TelegramBotSender>();
builder.Services.AddTransient<ITelegramChannelReader, TelegramChannelReader>();
builder.Services.AddTransient<ISummarizationService, GoogleGeminiSummarizationService>();// <--   

// 6.    Worker Service (    )
builder.Services.AddHostedService<Worker>();

// ---     ---
var host = builder.Build();
host.Run();